#include <p32xxxx.h>
#include <plib.h>
#include <stdio.h>
#include "System.h"

#define Timer1Tick ( PBFrequency / 256 / Timer1TogglesPerSec )
#define Timer1TogglesPerSec 10
#define Timer3Tick ( PBFrequency / 256 / Timer3TogglesPerSec )
#define Timer3TogglesPerSec ????

unsigned int VR1Value = 0;
unsigned int VR2Value = 0;
volatile unsigned int ADDone = 0;

void main(void)
{
    InitializeSystem();

    // 使用XC32 I/O Prot Function將LED(RD2, RD5)對應之I/O設定成數位輸出，
    // 並指定I/O Port狀態。
    mPORTDSetPinsDigitalOut(BIT_2 | BIT_5);
    mPORTDClearBits(BIT_2 | BIT_5);

    // 使用XC32 I/O Prot Function將Button(RF4)對應之I/O設定成數位輸入，
    mPORTFSetPinsDigitalIn(BIT_4);

    // 使用XC32 OpenTimer1 Function 設定Timer1。
    OpenTimer1(T1_ON | T1_GATE_OFF | T1_PS_1_256 |
               T1_SOURCE_INT, Timer1Tick);

    // 使用XC32 ConfigIntTimer1 Function啟用Timer1中斷
    // 並指定其主要中斷優先權為4，次要優先權為0。
    ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_4 | T1_INT_SUB_PRIOR_0);

    // To Do : 使用XC32 OpenADC10 Function設定ADC。(ADC_CLK_MANUAL -> ADC_CLK_TMR)
    OpenADC10(ADC_MODULE_ON | ADC_IDLE_CONTINUE | ADC_FORMAT_INTG16 | ADC_CLK_???? | ADC_AUTO_SAMPLING_ON | ADC_SAMP_OFF,
              ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_ON | ADC_SAMPLES_PER_INT_2 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF,
              ADC_SAMPLE_TIME_31 | ADC_CONV_CLK_SYSTEM | ADC_CONV_CLK_32Tcy,
              ENABLE_AN19_ANA | ENABLE_AN18_ANA,
              SKIP_SCAN_AN0 | SKIP_SCAN_AN1 | SKIP_SCAN_AN2 | SKIP_SCAN_AN3 |
              SKIP_SCAN_AN4 | SKIP_SCAN_AN5 | SKIP_SCAN_AN6 | SKIP_SCAN_AN7 |
              SKIP_SCAN_AN8 | SKIP_SCAN_AN9 | SKIP_SCAN_AN10 | SKIP_SCAN_AN11 |
              SKIP_SCAN_AN12 | SKIP_SCAN_AN13 | SKIP_SCAN_AN14 | SKIP_SCAN_AN15 |
              SKIP_SCAN_AN16 | SKIP_SCAN_AN17 |
              SKIP_SCAN_AN20 | SKIP_SCAN_AN21 | SKIP_SCAN_AN22 | SKIP_SCAN_AN23 |
              SKIP_SCAN_AN24 | SKIP_SCAN_AN25 | SKIP_SCAN_AN26 | SKIP_SCAN_AN27 |
              SKIP_SCAN_AN28 | SKIP_SCAN_AN29 | SKIP_SCAN_AN30);

    // To Do : 使用XC32 OpenTimer3 Function 設定Timer3。
    OpenTimer????( ???? | ???? | ... | T3_SOURCE_????, ???? );

    // To Do : 使用XC32 ConfigIntADC10 Function啟用ADC中斷
    //		   並指定其主要中斷優先權為5，次要優先權為0。
    ConfigIntADC10(ADC_INT_???? | ADC_INT_PRI_???? | ADC_INT_SUB_PRI_???? );

    // 使用XC32 INTEnableSystemMultiVectoredInt Function
    // 設定為PIC32為多重中斷向量模式。
    INTEnableSystemMultiVectoredInt();

    while (1)
    {
        // 判斷ADC1中斷旗標。
        if (ADDone)
        {
            ADDone = 0;
            
            // 透過UART，輸出ADC數值。
            printf("Lab8 : VR1 Value = %4d , VR2 Value = %4d\r\n", VR1Value, VR2Value);
        }

        // 使用XC32 I/O Prot Function 讀取按鍵(BT1, RF4)狀態，
        // 並反應到LED(D11, RD5)上。
        if (mPORTFReadBits(BIT_4))
            mPORTDClearBits(BIT_5);
        else
            mPORTDSetBits(BIT_5);
    }
}

// 建立Timer1的中斷服務函式，並指定主要中斷優先權為4。
void __ISR(_TIMER_1_VECTOR, ipl4) ISR_Timer1(void)
{
    // 清除Timer中斷旗標。
    INTClearFlag(INT_T1);

    // 使用XC32 I/O Prot Function改變IO Port(RD2)狀態(Toggle)。
    mPORTDToggleBits(BIT_2);
}

// To Do : 建立ADC的中斷服務函式，並指定主要中斷優先權為5。
void __ISR(_????_VECTOR, ipl5) ISR_ADC(void)
{
    // To Do : 自ADC Buffer0, Buffer1讀取ADC轉換之數值。
    VR2Value = (unsigned int) ReadADC10(????);
    VR1Value = (unsigned int) ReadADC10(????);

    // 清除ADC1中斷旗標
    INTClearFlag(INT_AD1);

    ADDone = 1;
}
