#include <p32xxxx.h>
#include <plib.h>
#include <stdio.h>
#include "System.h"

#define Timer1Tick ( PBFrequency / 256 / Timer1TogglesPerSec )
#define Timer1TogglesPerSec 2
#define Timer2Tick ( PBFrequency / 1 / Timer2TogglesPerSec )
#define Timer2TogglesPerSec 1000
#define OCPWMPeriod Timer2Tick

unsigned int VR1Value = 0;
volatile unsigned char UART_IsReceived = 0;
volatile unsigned char UART_ReceiveData = 0;
unsigned int DutyValue = 100;

void main(void)
{
    InitializeSystem();

    // To Do : 使用XC32 I/O Prot Function將LED(RD2, RD5, RD8, RD9, RD10)對應之I/O設定成數位輸出，
    //         並指定I/O Port狀態。
    mPORTDSetPinsDigitalOut(BIT_2 | BIT_5 | BIT_8 | BIT_9 | BIT_10);
    mPORTDClearBits(BIT_2 | BIT_5 | BIT_8 | BIT_9 | BIT_10);

    // 使用XC32 I/O Prot Function將Button(RF4)對應之I/O設定成數位輸入，
    mPORTFSetPinsDigitalIn(BIT_4);

    // 使用XC32 OpenTimer1 Function 設定Timer1。
    OpenTimer1(T1_ON | T1_GATE_OFF | T1_PS_1_256 |
               T1_SOURCE_INT, Timer1Tick);

    // 使用XC32 ConfigIntTimer1 Function啟用Timer1中斷
    // 並指定其主要中斷優先權為4，次要優先權為0。
    ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_4 | T1_INT_SUB_PRIOR_0);

    // 使用XC32 OpenADC10 Function設定ADC。
    OpenADC10(ADC_MODULE_ON | ADC_IDLE_CONTINUE | ADC_FORMAT_INTG16 | ADC_CLK_MANUAL | ADC_AUTO_SAMPLING_ON | ADC_SAMP_OFF,
              ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_OFF | ADC_SAMPLES_PER_INT_1 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF,
              ADC_SAMPLE_TIME_31 | ADC_CONV_CLK_SYSTEM | ADC_CONV_CLK_32Tcy,
              ENABLE_AN19_ANA, SKIP_SCAN_ALL);

    // 使用XC32 SetChanADC10 Function設定AD正確的正,負通道( (+)AN19 , (-)VREF- )。
    SetChanADC10(ADC_CH0_POS_SAMPLEA_AN19 | ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEB_AN0 | ADC_CH0_NEG_SAMPLEB_NVREF);

    // To Do : 使用XC32 OpenTimer2 Function 設定Timer2, 作為OCx的Time Base。
    OpenTimer2(T2_ON | T2_GATE_OFF | T2_PS_1_1 |
               T2_SOURCE_INT, Timer2Tick);

    // To Do : 使用XC32 PPS Function設定OC3。
    PPSUnLock;
    PPSOutput(1, RPD10, OC3); //Assign OC3
    PPSLock;

    // To Do : 使用XC32 OC Function進行設定。
    OpenOC3(OC_ON | OC_TIMER_MODE16 | OC_TIMER2_SRC | OC_PWM_FAULT_PIN_DISABLE,
            OCPWMPeriod, OCPWMPeriod);

    // 使用XC32 INTEnableSystemMultiVectoredInt Function
    // 設定為PIC32為多重中斷向量模式。
    INTEnableSystemMultiVectoredInt();

    while (1)
    {
        // 判斷ADC1中斷旗標。
        if (INTGetFlag(INT_AD1))
        {
            // 自ADC Buffer0讀取ADC轉換之數值。
            VR1Value = (unsigned int) ReadADC10(0);

            // 清除ADC1中斷旗標
            INTClearFlag(INT_AD1);

            // 透過UART，輸出ADC數值。
            printf("Lab11 : VR1 Value = %4d , OC3RS(Duty):%5d , PR2(Period):%5d , PWM Duty : %3d%% \r\n",
                   VR1Value , OC3RS , PR2 , DutyValue);
        }

        // 判斷UART2是否接收到資料
        if (UART_IsReceived)
        {
            UART_IsReceived = 0;

            switch (UART_ReceiveData)
            {
            case '+':
                mPORTDToggleBits(BIT_8);

                if (DutyValue < 100)
                    DutyValue += 10;
                break;

            case '-':
                mPORTDToggleBits(BIT_9);

                if (DutyValue > 0)
                    DutyValue -= 10;
                break;
            }

            // To Do : 使用XC32 OC Function設定PWM Duty。
            if (DutyValue == 100)
                SetDCOC3PWM(OCPWMPeriod + 1);
            else
                SetDCOC3PWM(((long) DutyValue * OCPWMPeriod) / 100);

            printf("Received : %1c\r\n", UART_ReceiveData);
        }

        // 使用XC32 I/O Prot Function 讀取按鍵(BT1, RF4)狀態，
        // 並反應到LED(D11, RD5)上。
        if (mPORTFReadBits(BIT_4))
            mPORTDClearBits(BIT_5);
        else
            mPORTDSetBits(BIT_5);
    }
}

// 建立Timer1的中斷服務函式，並指定主要中斷優先權為4。
void __ISR(_TIMER_1_VECTOR, ipl4) ISR_Timer1(void)
{
    // 清除Timer中斷旗標。
    INTClearFlag(INT_T1);

    // 使用XC32 ConvertADC10 Function啟動ADC的轉換。
    ConvertADC10();

    // 使用XC32 I/O Prot Function改變IO Port(RD2)狀態(Toggle)。
    mPORTDToggleBits(BIT_2);
}

// 建立UART2的中斷服務函式，並指定主要中斷優先權為5。
void __ISR(_UART_2_VECTOR, ipl5) ISR_UART2(void)
{
    // U2E & U2TX & U2RX 共用向量, 必須確認是否為U2RX。
    if (INTGetFlag(INT_U2RX))
    {
        // 使用XC32 UART Function讀取接收到的資料(放入變數UART_ReceiveData中)。
        // 注意 : 此處資料僅簡單使用單一變數儲存，若資料瞬間流量大時，
        //        必須透過其他軟體機制輔助，以免資料遺失，
        //        例如:建立一個Data Queue來存放收到的資料。
        UART_ReceiveData = UARTGetDataByte(UART2);

        // 清除U2RX中斷旗標。
        INTClearFlag(INT_U2RX);

        UART_IsReceived = 1;
    }

    // 正常狀態下, 不應該進入此處, 若進入此處, 代表UART有非預期中斷發生。
    else
    {
        while (1);
    }
}
