#include <p32xxxx.h>
#include <plib.h>
#include <stdio.h>
#include "System.h"

void InitializeSystem(void)
{
    // Disable the JTAG function
    DDPCONbits.JTAGEN = 0;

    // To Do : 使用XC32提供的Oscillator On Fly Change Function來設定時脈
    //         將CPU時脈設定為80MHz,
    //         時脈來源可採用外部時脈(APP028-1外部時脈為8MHz)。
    //         周邊時脈與系統時脈的比率設成1:8。
    //         8MHz / 2 * 20 / 1 = 80MHz。

    // Oscillator On-Fly-Change Function
    // OSCConfig( Clock Source , PLL Multiplier , PLL Postscaler , FRC Divisor );
    // 以PLLIDIV 設為除2, Primary Crystal = 8MHz為例: (*PLLIDIV無法動態修改)

    // 動態改變MCU的系統時脈為Internal Fast RC / 1 (8MHz)。
    // Sys. Clock =  FRC = 8 MHz
    // OSCConfig(OSC_FRC, 0, 0, 0);

    // 動態改變MCU的系統時脈為Internal Fast RC / 2 (8MHz)。
    // Sys. Clock =  FRC / 2 = 4 MHz
    // OSCConfig(OSC_FRC_DIV, 0, 0, OSC_FRC_POST_2);

    // 動態改變MCU的系統時脈為Primary Oscillator With PLL / 2 (40MHz)。
    // Sys. Clock =  ((FRC / 2) * 20) / 2 = 40 MHz
    // OSCConfig(OSC_FRC_PLL, OSC_PLL_MULT_20, OSC_PLL_POST_2, 0);

    // 動態改變MCU的系統時脈為Primary Oscillator With PLL / 1 (80MHz)。
    // Sys. Clock =  ((PRI / 2) * 20) / 1 = 80 MHz
    OSCConfig(OSC_POSC_PLL, OSC_PLL_MULT_20, OSC_PLL_POST_1, 0);

    // 周邊時脈(PB Clock)與系統時脈(System Clock)的比率也可以動態切換。

    // 動態改變System Clock跟PB Clock的比率為1:1。
    // System Freq. / 1 = PB Freq.
    // OSCSetPBDIV(OSC_PB_DIV_1);

    // 動態改變System Clock跟PB Clock的比率為1:2。
    // System Freq. / 2 = PB Freq.
    // OSCSetPBDIV(OSC_PB_DIV_2);

    // 動態改變System Clock跟PB Clock的比率為1:4。
    // System Freq. / 4 = PB Freq.
    // OSCSetPBDIV(OSC_PB_DIV_4);

    // 動態改變System Clock跟PB Clock的比率為1:8。
    // System Freq. / 8 = PB Freq.
    OSCSetPBDIV(OSC_PB_DIV_8);

    // 使用XC32的SYSTEM Config函數將Wait State與Prefetch最佳化。
    SYSTEMConfig(SystemFrequency, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);

    // 設定U2RX, U2TX。
    PPSUnLock;
    PPSOutput(2, RPF0, U2TX); //Assign U2TX
    PPSInput(1, U2RX, RPF1); //Assign U2RX
    PPSLock;

    // 設定UART ( 9600 , N , 8 , 1 )。
    UARTConfigure(UART2, UART_ENABLE_PINS_TX_RX_ONLY);
    UARTSetFifoMode(UART2, UART_INTERRUPT_ON_TX_NOT_FULL | UART_INTERRUPT_ON_RX_NOT_EMPTY);
    UARTSetLineControl(UART2, UART_DATA_SIZE_8_BITS | UART_PARITY_NONE | UART_STOP_BITS_1);
    UARTSetDataRate(UART2, PBFrequency, 9600);
    UARTEnable(UART2, UART_ENABLE | UART_PERIPHERAL | UART_TX | UART_RX);

    //setbuf(stdout, NULL);
}
