#include <p32xxxx.h>
#include <plib.h>
#include <stdio.h>
#include "System.h"

#define Timer1Tick ( PBFrequency / 256 / Timer1TogglesPerSec )
#define Timer1TogglesPerSec 10

unsigned int VR1Value = 0;

void main(void)
{
    InitializeSystem();

    // 使用XC32 I/O Prot Function將LED(RD2, RD5)對應之I/O設定成數位輸出，
    // 並指定I/O Port狀態。
    mPORTDSetPinsDigitalOut(BIT_2 | BIT_5);
    mPORTDClearBits(BIT_2 | BIT_5);

    // 使用XC32 I/O Prot Function將Button(RF4)對應之I/O設定成數位輸入，
    mPORTFSetPinsDigitalIn(BIT_4);

    // 使用XC32 OpenTimer1 Function 設定Timer1。
    OpenTimer1(T1_ON | T1_GATE_OFF | T1_PS_1_256 |
               T1_SOURCE_INT, Timer1Tick);

    // 使用XC32 ConfigIntTimer1 Function啟用Timer1中斷
    // 並指定其主要中斷優先權為4，次要優先權為0。
    ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_4 | T1_INT_SUB_PRIOR_0);

    // To Do : 使用XC32 OpenADC10 Function設定ADC。
    OpenADC10(ADC_MODULE_ON | ADC_IDLE_CONTINUE | ADC_FORMAT_INTG16 | ADC_CLK_MANUAL | ADC_AUTO_SAMPLING_ON | ADC_SAMP_OFF,
              ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_OFF | ADC_SAMPLES_PER_INT_1 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF,
              ADC_SAMPLE_TIME_31 | ADC_CONV_CLK_SYSTEM | ADC_CONV_CLK_32Tcy,
              ENABLE_AN19_ANA, SKIP_SCAN_ALL);

    //  To Do : 使用XC32 SetChanADC10 Function設定AD正確的正,負通道( (+)AN19 , (-)VREF- )。
    SetChanADC10(ADC_CH0_POS_SAMPLEA_AN19 | ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEB_AN0 | ADC_CH0_NEG_SAMPLEB_NVREF);

    // 使用XC32 INTEnableSystemMultiVectoredInt Function
    // 設定為PIC32為多重中斷向量模式。
    INTEnableSystemMultiVectoredInt();

    while (1)
    {
        // To Do : 判斷ADC1中斷旗標。
        if (INTGetFlag(INT_AD1))
        {
            // To Do : 自ADC Buffer0讀取ADC轉換之數值。
            VR1Value = (unsigned int) ReadADC10(0);

            // To Do : 清除ADC1中斷旗標
            INTClearFlag(INT_AD1);

            // 透過UART，輸出ADC數值。
            printf("Lab6 : VR1 Value = %4d\r\n" , VR1Value );
        }
        
        // 使用XC32 I/O Prot Function 讀取按鍵(BT1, RF4)狀態，
        // 並反應到LED(D11, RD5)上。
        if (mPORTFReadBits(BIT_4))
            mPORTDClearBits(BIT_5);
        else
            mPORTDSetBits(BIT_5);
    }
}

// 建立Timer1的中斷服務函式，並指定主要中斷優先權為4。
void __ISR(_TIMER_1_VECTOR, ipl4) ISR_Timer1(void)
{
    // 清除Timer中斷旗標。
    INTClearFlag(INT_T1);

    // To Do : 使用XC32 ConvertADC10 Function啟動ADC的轉換。
    ConvertADC10();

    // 使用XC32 I/O Prot Function改變IO Port(RD2)狀態(Toggle)。
    mPORTDToggleBits(BIT_2);
}
